%*************************************************************************%
%                          U20 Friction Tool                              %
%                                                                         %
% This file is part of the U20 Friction Tool. The U20 Friction tool is    %
% free software. You can use, redistribute and modify it. If you publish  %
% results obtained with this tool (or any tool based on this code),       %
% please cite the following article:                                      %
%                                                                         %
% R.H.W. ten Thije, R. Akkerman, L. van der Meer, M. P. Ubbink,           %
% Tool-ply friction in thermoplastic composite forming,                   %
% International Journal of Material Forming, Volume 1, p. 953-956 (2008)  %
%                                                                         %
% This library is distributed in the hope that it will be useful, but     %
% WITHOUT ANY WARRANTY.                                                   %
%                                                                         %
% Copyright (C) 2009 Rene ten Thije.                                      %
% Developed at the University of Twente, the Netherlands.                 %
% Chair of Production Technology, www.pt.ctw.utwente.nl                   %
% info: r.tenthije@utwente.nl / r.akkerman@utwente.nl                     %
%                                                                         %
%*************************************************************************%

function [Par] = solveREGeo(Material,Geo,Par)

MaxNrm  = Par.MaxNrm;
MaxIter = Par.MaxIter;
N_Goal  = Par.N_Goal;
h_min   = Par.h_min;
h_min   = Par.lowlimit;

nrm      = inf;
prev_nrm = inf;
iter = 0;
disp('Iterating to find h0.')
while ((nrm > MaxNrm) && (iter < MaxIter))
    iter = iter +1;

    % ********* find the current solution *********************************
    N  = 0;
    Ff = 0;
    nGeo = Geo.nGeo;
    for iGeo = 1:nGeo       % loop over the contours
        Contour = Geo.Contour{iGeo};
        W       = Geo.Width{iGeo};
        nCont   = Geo.nCont{iGeo};

        % solve the reynoldsequation
        [p,x0,rel_err,iter_rey] = solveReynolds(Contour,Material,Par);
        [Nc,Ffc] = giveNandFw(p,Contour,Material,Par);
        N  = N + nCont*W*Nc;
        Ff = Ff + nCont*W*Ffc;
        Sol{iGeo}.p=p;
        Sol{iGeo}.x0=x0;
        Sol{iGeo}.rel_err=rel_err;
        Sol{iGeo}.iter_rey=iter_rey;
    end

    % calculate the accuracy of the current solution
    nrm = abs((N-N_Goal)/N_Goal);

    % if this iteration did not reduce the norm, then the dh0 was probably
    % to big. Reduce the dh0 until the norm reduces
    in_loop_iter = 0;
    max_in_loop_iter = 100;
    while ((nrm > prev_nrm) && (in_loop_iter<max_in_loop_iter))
        in_loop_iter = in_loop_iter+1;
        dh0 = dh0/2;
        h0  = h00 + dh0;
        % do not allow a value below h_min
        if h0 < h_min
            h0 = h_min;
        end
        % set h0
        Par.h0 = h0;

        N  = 0;
        Ff = 0;
        for iGeo = 1:nGeo
            Contour = Geo.Contour{iGeo};
            W       = Geo.Width{iGeo};
            nCont   = Geo.nCont{iGeo};

            [p,x0,rel_err,iter_rey] = solveReynolds(Contour,Material,Par);
            [Nc,Ffc] = giveNandFw(p,Contour,Material,Par);
            N  = N + nCont*W*Nc;
            Ff = Ff + nCont*W*Ffc;
            Sol{iGeo}.p=p;
            Sol{iGeo}.x0=x0;
            Sol{iGeo}.rel_err=rel_err;
            Sol{iGeo}.iter_rey=iter_rey;
        end

        % check the normal force
        nrm = abs((N-N_Goal)/N_Goal);
    end

    % **** show some information about the iterative process on screen  ***
    if in_loop_iter >0
        ToScreen=['                   Diverging process. ',...
            num2str(in_loop_iter),' inner loop(s) to get convergence again.'];
        disp(ToScreen);pause(0.05);
    end
    ToScreen=['  Iteration nr: ',num2str(iter),'. N : ',num2str(N,'%14.6E'),...
        ' h0: ',num2str(Par.h0,'%14.6E'),' norm: ',num2str(nrm,'%14.6E')];
    disp(ToScreen);pause(0.05);

    % *********  find the tangent and make a new guess for h0***********
    if nrm > MaxNrm
        Par_dx = Par;
        Par_dx.h0 = Par_dx.h0 + sqrt(eps);

        N_dx = 0;
        for iGeo = 1:nGeo
            Contour = Geo.Contour{iGeo};
            W       = Geo.Width{iGeo};
            nCont   = Geo.nCont{iGeo};
            [p,x0,rel_err,iter_rey] = solveReynolds(Contour,Material,Par_dx);
            [Nc,Ffc] = giveNandFw(p,Contour,Material,Par);
            N_dx  = N_dx + nCont*W*Nc;
        end
        dNdh0 = (N_dx-N)/sqrt(eps);
        dh0   = (N_Goal-N)/dNdh0;
        h00   = Par.h0;
        h0    = h00 + dh0; % new estimate for h0

        % do not allow a value below h_min
        if h0 < h_min
            h0 = h_min;
        end

        % set the new value
        Par.h0 = h0;
    end
    
    prev_nrm = nrm;
end
Par.N  = N;
Par.Ff = Ff;