%*************************************************************************%
%                          U20 Friction Tool                              %
%                                                                         %
% This file is part of the U20 Friction Tool. The U20 Friction tool is    %
% free software. You can use, redistribute and modify it. If you publish  %
% results obtained with this tool (or any tool based on this code),       %
% please cite the following article:                                      %
%                                                                         %
% R.H.W. ten Thije, R. Akkerman, L. van der Meer, M. P. Ubbink,           %
% Tool-ply friction in thermoplastic composite forming,                   %
% International Journal of Material Forming, Volume 1, p. 953-956 (2008)  %
%                                                                         %
% This library is distributed in the hope that it will be useful, but     %
% WITHOUT ANY WARRANTY.                                                   %
%                                                                         %
% Copyright (C) 2009 Rene ten Thije.                                      %
% Developed at the University of Twente, the Netherlands.                 %
% Chair of Production Technology, www.pt.ctw.utwente.nl                   %
% info: r.tenthije@utwente.nl / r.akkerman@utwente.nl                     %
%                                                                         %
%*************************************************************************%

function [p,x0,rel_err,iter] = solveReynolds(contour,material,par)

% set values 
h0 = par.h0;       % film thickness at x = 0
T  = par.T;        % temperature in deg C
v  = par.v;        % velocity
hc = contour.y;
x  = contour.x;
dx = contour.dx;
n  = contour.n;

% the derivative of the contour
dhdx=centr_diff(hc,dx);

% the film thickness
h=hc+h0;

% shear rate
gam_dot=v./h;
eta  =give_eta(material,T,gam_dot);% eta

% the vectors
alpha=3./h.*dhdx;  % - 1./eta.*centr_diff(eta,dx); last term is negligable
beta=6*eta*v./(h.^3).*dhdx;
v1=1/dx^2*ones(1,n);
v2=1/(2*dx)*alpha;
DiagVec = -2*v1';
LowDiagVec= [v1(2:end)'-v2(2:end)';0];
UppDiagVec= [0; v1(1:end-1)'+v2(1:end-1)'];
SpMat=spdiags([LowDiagVec DiagVec UppDiagVec], -1:1, n, n);

Stop=0;
iter=0;
MaxIter=25;
ix0=1;
while ~Stop
    iter=iter+1;
    ix0_old=ix0;
    MSolve=SpMat;
    MSolvedx=SpMat;
    ySolve=beta;
    ySolvedx=beta;

    % constrainted degrees of freedom
    p0=[1 ix0];
    for i=1:length(p0)
        indx=p0(i);
        ySolve(indx)=0;
        MSolve(indx,:)=0;
        MSolve(:,indx)=0;
        MSolve(indx,indx)=1;
    end

    % for the numerical derivative, used to find x0
    p0dx=[1 ix0+1];
    for i=1:length(p0)
        indx=p0dx(i);
        ySolvedx(indx+1)=0;
        MSolvedx(indx+1,:)=0;
        MSolvedx(:,indx+1)=0;
        MSolvedx(indx+1,indx+1)=1;
    end

    % solve the system
    p=MSolve\ySolve';
    pdx=MSolvedx\ySolvedx';

    %plot(p);pause(1)
    
    % make a good guess in the first loop
    if iter==1
        [minp,ix0]=min(p);
        % estimate the error
        d3p=-p(1:end-4)+2*p(2:end-3)-2*p(4:end-1)+p(5:end);
        d3pdx3=d3p./dx^3;
        M3=max(abs(d3pdx3));
        err=M3*dx^2/6;
        rel_err=err/max(1/dx*abs(diff(p)));
%        disp(['relative error in the derivative: ',num2str(100*rel_err),'%']);
    else
        % new guess for x0
        dpdx_x0  = (p(ix0)-p(ix0-1))/dx;
        dpdx_x0dx= (pdx(ix0+1)-pdx(ix0))/dx;
        ddpdxdx0 = (dpdx_x0dx-dpdx_x0)/dx;
        dx0      =-dpdx_x0/ddpdxdx0;
        dx0max   =(x(end)-x(1))/10;
        if abs(dx0)>dx0max
            dx0=sign(dx0)*dx0max;
        end
        x0=x(ix0)+dx0;
        [delta_x,ix0]=min(abs(x-x0));
        if ix0==1
            ix0=2;
        end
    end
    
    if abs(ix0-ix0_old)<=1
        Stop=1;
    end

    if iter>MaxIter
        Stop=1;
    end
end
p(ix0:end)=0;